@@ -17,12 +17,17 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 import chess
-import collections
 import signal
 import subprocess
 import logging
 import threading
 import concurrent.futures
+import sys
+
+try:
+    import backport_collections as collections
+except ImportError:
+    import collections
 
 try:
     import queue
@@ -102,6 +107,23 @@ class OptionMap(collections.MutableMapping):
 
 
 class InfoHandler(object):
+    """
+    Chess engines may send information about their calculations with the
+    *info* command. You can register info handlers to be asynchronously
+    notified whenever the engine sends more information.
+
+    >>> # Register a standard info handler.
+    >>> info_handler = chess.uci.InfoHandler()
+    >>> engine.info_handlers.append(info_handler)
+
+    You would usually subclass the *InfoHandler* class.
+
+    >>> class MyHandler(chess.uci.InfoHandler):
+    ...     def post_info(self):
+    ...         # Called whenever a complete *info* line has been processed.
+    ...         super(MyHandler, self).post_info()
+    ...         print(self.info)
+    """
     def __init__(self):
         self.lock = threading.Lock()
 
@@ -1008,6 +1030,9 @@ class Engine(object):
         :raises: :exc:`~chess.uci.EngineStateException` if the engine is still
             calculating.
         """
+        # Work on a local copy.
+        board = board.copy()
+
         # Raise if this is called while the engine is still calculating.
         with self.state_changed:
             if not self.idle:
@@ -1058,7 +1083,7 @@ class Engine(object):
                 builder.append(board.uci(move, chess960=self.uci_chess960))
                 board.push(move)
 
-        self.board = board.copy()
+        self.board = board
 
         def command():
             with self.semaphore:
@@ -1333,11 +1358,11 @@ def popen_engine(command, engine_cls=Engine):
 
 def spur_spawn_engine(shell, command, engine_cls=Engine):
     """
-    Spwans a remote engine using a `Spur`_ shell.
+    Spawns a remote engine using a `Spur`_ shell.
 
     >>> import spur
     >>> shell = spur.SshShell(hostname="localhost", username="username", password="pw")
-    >>> engine = chess.uci.spur_spwan_engine(shell, ["/usr/games/stockfish"])
+    >>> engine = chess.uci.spur_spawn_engine(shell, ["/usr/games/stockfish"])
     >>> engine.uci()
 
     .. _Spur: https://pypi.python.org/pypi/spur
